﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;




using System.Windows.Media;

using System.Windows.Media.Media3D;

namespace TextTransform
{
    public partial class TextTransformer : UserControl
    {
        Rect borderRect = Rect.Empty;

        public TextTransformer()
        {
            InitializeComponent();
            LayoutUpdated += TextTransformerLayoutUpdated;
        }

        void TextTransformerLayoutUpdated(object sender, EventArgs e)
        {
            if (borderRect.IsEmpty)
            {
                Canvas.SetLeft(thumbUR, border.ActualWidth);
                Canvas.SetTop(thumbLL, border.ActualHeight);
                Canvas.SetLeft(thumbLR, border.ActualWidth);
                Canvas.SetTop(thumbLR, border.ActualHeight);

                borderRect = new Rect(0, 0, border.ActualWidth, border.ActualHeight);
            }
        }

        void OnThumbDragDelta(object sender, DragDeltaEventArgs args)
        {
            Thumb thumb = sender as Thumb;
            Canvas.SetLeft(thumb, Canvas.GetLeft(thumb) + args.HorizontalChange);
            Canvas.SetTop(thumb, Canvas.GetTop(thumb) + args.VerticalChange);

            Matrix3D matrix =
                CalculateNewTransform(borderRect,
                    new Point(Canvas.GetLeft(thumbUL), Canvas.GetTop(thumbUL)),
                    new Point(Canvas.GetLeft(thumbUR), Canvas.GetTop(thumbUR)),
                    new Point(Canvas.GetLeft(thumbLL), Canvas.GetTop(thumbLL)),
                    new Point(Canvas.GetLeft(thumbLR), Canvas.GetTop(thumbLR)));

            matrixProjection.ProjectionMatrix = matrix;
        }

        Matrix3D CalculateNewTransform(Rect rect, Point ptUL, Point ptUR, Point ptLL, Point ptLR)
        {
            // Scale and translate transform
            Matrix3D S = new Matrix3D()
            {
                M11 = 1 / rect.Width,
                M22 = 1 / rect.Height,
                OffsetX = -rect.Left / rect.Width,
                OffsetY = -rect.Top / rect.Height
            };

            // See http://www.charlespetzold.com/blog/2007/08/250638.html for math that follows

            // Affine transform
            Matrix3D A = new Matrix3D()
            {
                OffsetX = ptUL.X,
                OffsetY = ptUL.Y,
                M11 = (ptUR.X - ptUL.X),
                M12 = (ptUR.Y - ptUL.Y),
                M21 = (ptLL.X - ptUL.X),
                M22 = (ptLL.Y - ptUL.Y)
            };

            // Non-Affine transform
            Matrix3D B = new Matrix3D();
            double den = A.M11 * A.M22 - A.M12 * A.M21;
            double a = (A.M22 * ptLR.X - A.M21 * ptLR.Y + A.M21 * A.OffsetY - A.M22 * A.OffsetX) / den;
            double b = (A.M11 * ptLR.Y - A.M12 * ptLR.X + A.M12 * A.OffsetX - A.M11 * A.OffsetY) / den;

            B.M11 = a / (a + b - 1);
            B.M22 = b / (a + b - 1);
            B.M14 = B.M11 - 1;
            B.M24 = B.M22 - 1;

            // Product of three transforms
            return S * B * A;
        }
    }
}
